@@ -2,8 +2,8 @@
 #
 # Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
 #     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004,
-#     2005, 2006, 2007, 2008, 2009, 2010, 2011 Massachusetts Institute
-#     of Technology
+#     2005, 2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014,
+#     2015, 2016, 2017, 2018 Massachusetts Institute of Technology
 #
 # This file is part of MIT/GNU Scheme.
 #
@@ -33,8 +33,13 @@ else
     exit 1
 fi
 
-run_cmd "${EXE}" --band runtime.com --heap 6000 --stack 200 <<EOF
+run_cmd "${EXE}" --batch-mode --library lib --band x-compiler.com <<EOF
 (begin
   (load "etc/compile.scm")
-  (c-prepare))
+  (in-liarc
+   (lambda ()
+     (compile-boot-dirs c-compile-dir))))
 EOF
+
+# distclean will not get these...
+run_cmd ./Stage.sh remove X
