@@ -2,7 +2,8 @@
 
 Copyright (C) 1986, 1987, 1988, 1989, 1990, 1991, 1992, 1993, 1994,
     1995, 1996, 1997, 1998, 1999, 2000, 2001, 2002, 2003, 2004, 2005,
-    2006, 2007, 2008, 2009, 2010 Massachusetts Institute of Technology
+    2006, 2007, 2008, 2009, 2010, 2011, 2012, 2013, 2014, 2015, 2016,
+    2017, 2018 Massachusetts Institute of Technology
 
 This file is part of MIT/GNU Scheme.
 
@@ -27,19 +28,15 @@ USA.
 
 #include "scheme.h"
 #include "prims.h"
-#include "winder.h"
 #include "history.h"
 #include "syscall.h"
-
-#ifdef __OS2__
-  extern void OS2_handle_attention_interrupt (void);
-#endif
+#include "bignmint.h"
 
 SCHEME_OBJECT * history_register;
 unsigned long prev_restore_history_offset;
 
 static SCHEME_OBJECT copy_history (SCHEME_OBJECT);
-static void error_death (long, char *) NORETURN;
+static void error_death (long, const char *) NORETURN;
 
 /* Helper procedures for setup_interrupt, which follows. */
 
@@ -107,8 +104,8 @@ initialize_interrupt_mask_vector (void)
   return (v);
 }
 
-/* setup_interrupt is called from the Interrupt macro to do all of the
-   setup for calling the user's interrupt routines. */
+/* setup_interrupt is called from the SIGNAL_INTERRUPT macro to do all
+   of the setup for calling the user's interrupt routines. */
 
 void
 setup_interrupt (unsigned long masked_interrupts)
@@ -120,13 +117,6 @@ setup_interrupt (unsigned long masked_interrupts)
   unsigned long interrupt_mask;
   SCHEME_OBJECT interrupt_handler;
 
-#ifdef __OS2__
-  if ((1UL << interrupt_number) == INT_Global_1)
-    {
-      OS2_handle_attention_interrupt ();
-      abort_to_interpreter (PRIM_POP_RETURN);
-    }
-#endif
   if (!VECTOR_P (fixed_objects))
     {
       outf_fatal ("\nInvalid fixed-objects vector");
@@ -182,7 +172,7 @@ err_print (long error_code, outf_channel where)
 long death_blow;
 
 static void
-error_death (long code, char * message)
+error_death (long code, const char * message)
 {
   death_blow = code;
   outf_fatal ("\nMicrocode Error: %s.\n", message);
@@ -380,6 +370,17 @@ arg_integer (int arg_number)
   return (integer_to_long (object));
 }
 
+intmax_t
+arg_integer_to_intmax (int arg_number)
+{
+  SCHEME_OBJECT object = (ARG_REF (arg_number));
+  if (! (INTEGER_P (object)))
+    error_wrong_type_arg (arg_number);
+  if (! (integer_to_intmax_p (object)))
+    error_bad_range_arg (arg_number);
+  return (integer_to_intmax (object));
+}
+
 long
 arg_nonnegative_integer (int arg_number)
 {
@@ -398,6 +399,15 @@ arg_index_integer (int arg_number, long upper_limit)
   return (result);
 }
 
+intmax_t
+arg_index_integer_to_intmax (int arg_number, intmax_t upper_limit)
+{
+  intmax_t result = (arg_integer_to_intmax (arg_number));
+  if ((result < 0) || (result >= upper_limit))
+    error_bad_range_arg (arg_number);
+  return (result);
+}
+
 long
 arg_integer_in_range (int arg_number, long lower_limit, long upper_limit)
 {
@@ -480,6 +490,108 @@ arg_real_in_range (int arg_number, double lower_limit, double upper_limit)
   return (result);
 }
 
+/* The 32-bit FNV-1a hash, short for Fowler/Noll/Vo in honor of its
+   creators.  */
+
+uint32_t
+memory_hash (unsigned long length, const void * vp)
+{
+  const uint8_t * scan = ((const uint8_t *) vp);
+  const uint8_t * end = (scan + length);
+  uint32_t result = 2166136261U;
+  while (scan < end)
+    {
+      result ^= ((uint32_t) (*scan++));
+      result *= 16777619U;
+    }
+  return (result);
+}
+
+bool
+hashable_object_p (SCHEME_OBJECT object)
+{
+  switch (OBJECT_TYPE (object))
+    {
+    case TC_BYTEVECTOR:
+    case TC_CHARACTER_STRING:
+    case TC_INTERNED_SYMBOL:
+    case TC_UNINTERNED_SYMBOL:
+    case TC_BIG_FIXNUM:
+    case TC_BIG_FLONUM:
+    case TC_RATNUM:
+    case TC_COMPLEX:
+    case TC_LIST:
+    case TC_WEAK_CONS:
+    case TC_VECTOR:
+    case TC_CELL:
+      return (true);
+    default:
+      return (false);
+    }
+}
+
+uint32_t
+hash_object (SCHEME_OBJECT object)
+{
+  switch (OBJECT_TYPE (object))
+    {
+    case TC_BYTEVECTOR:
+    case TC_CHARACTER_STRING:
+      return (memory_hash ((BYTEVECTOR_LENGTH (object)),
+			   (BYTEVECTOR_POINTER (object))));
+
+    case TC_INTERNED_SYMBOL:
+    case TC_UNINTERNED_SYMBOL:
+      {
+	SCHEME_OBJECT name = (MEMORY_REF (object, SYMBOL_NAME));
+	return (memory_hash ((BYTEVECTOR_LENGTH (name)),
+			     (BYTEVECTOR_POINTER (name))));
+      }
+
+    case TC_BIG_FIXNUM:
+      return (memory_hash (((BIGNUM_LENGTH (object))
+			    * (sizeof (bignum_digit_type))),
+			   (BIGNUM_START_PTR (object))));
+
+    case TC_BIG_FLONUM:
+      return (memory_hash (((FLOATING_VECTOR_LENGTH (object))
+			    * (sizeof (double))),
+			   (FLOATING_VECTOR_LOC (object, 0))));
+
+    case TC_RATNUM:
+    case TC_COMPLEX:
+      return (combine_hashes ((hash_object (MEMORY_REF (object, 0))),
+			      (hash_object (MEMORY_REF (object, 1)))));
+
+    case TC_LIST:
+    case TC_WEAK_CONS:
+      return (combine_hashes ((hash_object (PAIR_CAR (object))),
+			      (hash_object (PAIR_CDR (object)))));
+
+    case TC_VECTOR:
+      {
+	const SCHEME_OBJECT * scan = (VECTOR_LOC (object, 0));
+	const SCHEME_OBJECT * end = (scan + (VECTOR_LENGTH (object)));
+	uint32_t result = 0;
+	while (scan < end)
+	  result = (combine_hashes (result, (hash_object (*scan++))));
+	return result;
+      }
+
+    case TC_CELL:
+      return (hash_object (MEMORY_REF (object, 0)));
+
+    default:
+      return (0);
+    }
+}
+
+uint32_t
+combine_hashes (uint32_t hash1, uint32_t hash2)
+{
+  return ((hash1 * 31) + hash2);
+}
+
 bool
 interpreter_applicable_p (SCHEME_OBJECT object)
 {
@@ -497,6 +609,16 @@ interpreter_applicable_p (SCHEME_OBJECT object)
 	object = (MEMORY_REF (object, ENTITY_OPERATOR));
 	goto tail_recurse;
       }
+
+    case TC_RECORD:
+      {
+	SCHEME_OBJECT applicator = record_applicator(object);
+	if (applicator == SHARP_F)
+	  return (false);
+	object = applicator;
+	goto tail_recurse;
+      }
+
 #ifdef CC_SUPPORT_P
     case TC_COMPILED_ENTRY:
       {
@@ -528,25 +650,37 @@ Do_Micro_Error (long error_code, bool from_pop_return_p)
   SCHEME_OBJECT handler = SHARP_F;
 
 #ifdef ENABLE_DEBUGGING_TOOLS
-  err_print (error_code, ERROR_OUTPUT);
-  if ((GET_RC == RC_INTERNAL_APPLY)
-      || (GET_RC == RC_INTERNAL_APPLY_VAL))
+  if (Print_Errors)
     {
-      SCHEME_OBJECT * sp = (STACK_LOC (CONTINUATION_SIZE));
-      Print_Expression ((sp[STACK_ENV_FUNCTION]), "Procedure was");
-      outf_error ("\n");
-      outf_error ("# of arguments: %lu\n",
-		  (APPLY_FRAME_HEADER_N_ARGS (sp[STACK_ENV_HEADER])));
-    }
-  else
-    {
-      Print_Expression (GET_EXP, "Expression was");
-      outf_error ("\n");
-      Print_Expression (GET_ENV, "Environment was");
+      err_print (error_code, ERROR_OUTPUT);
+      if ((GET_RC == RC_INTERNAL_APPLY)
+	  || (GET_RC == RC_INTERNAL_APPLY_VAL))
+	{
+	  Print_Expression (STACK_REF(CONTINUATION_SIZE + STACK_ENV_FUNCTION),
+			    "Procedure");
+	  outf_error ("\n");
+	  {
+	    int i, nargs = (APPLY_FRAME_HEADER_N_ARGS
+			    (STACK_REF(CONTINUATION_SIZE + STACK_ENV_HEADER)));
+	    for (i = 0; i < nargs; i += 1)
+	      {
+		outf_error ("Argument %d: ", i+1);
+		Print_Expression ((STACK_REF(CONTINUATION_SIZE
+					     + STACK_ENV_FIRST_ARG + i)), "");
+		outf_error ("\n");
+	      }
+	  }
+	}
+      else
+	{
+	  Print_Expression (GET_EXP, "Expression");
+	  outf_error ("\n");
+	  Print_Expression (GET_ENV, "Environment");
+	  outf_error ("\n");
+	}
+      Print_Return ("Return code");
       outf_error ("\n");
     }
-  Print_Return ("Return code");
-  outf_error ("\n");
 #endif
 
   if (Trace_On_Error)
@@ -759,7 +893,7 @@ copy_history (SCHEME_OBJECT hist_obj)
 {
   unsigned long space_left, vert_type, rib_type;
   SCHEME_OBJECT new_hunk, * last_hunk, * hist_ptr, * orig_hist, temp;
-  SCHEME_OBJECT * orig_rib, * source_rib, * rib_slot;
+  SCHEME_OBJECT * orig_rib, * source_rib, * rib_slot, * free;
 
   assert (HUNK3_P (hist_obj));
 
@@ -772,6 +906,7 @@ copy_history (SCHEME_OBJECT hist_obj)
   orig_hist = (OBJECT_ADDRESS (hist_obj));
   hist_ptr = orig_hist;
   last_hunk = (heap_end - 3);
+  free = Free;
 
   do
     {
@@ -780,14 +915,14 @@ copy_history (SCHEME_OBJECT hist_obj)
 	return (SHARP_F);
       space_left -= 3;
 
-      new_hunk = (MAKE_POINTER_OBJECT (vert_type, Free));
+      new_hunk = (MAKE_POINTER_OBJECT (vert_type, free));
       (last_hunk[HIST_NEXT_SUBPROBLEM]) = new_hunk;
 
-      (Free[HIST_PREV_SUBPROBLEM])
+      (free[HIST_PREV_SUBPROBLEM])
 	= (MAKE_POINTER_OBJECT ((OBJECT_TYPE (hist_ptr[HIST_PREV_SUBPROBLEM])),
 				last_hunk));
-      last_hunk = Free;
-      Free += 3;
+      last_hunk = free;
+      free += 3;
 
       /* Copy the rib. */
       temp = (hist_ptr[HIST_RIB]);
@@ -803,11 +938,11 @@ copy_history (SCHEME_OBJECT hist_obj)
 	    return (SHARP_F);
 	  space_left -= 3;
 
-	  (*rib_slot) = (MAKE_POINTER_OBJECT (rib_type, Free));
-	  (Free[RIB_EXP]) = (source_rib[RIB_EXP]);
-	  (Free[RIB_ENV]) = (source_rib[RIB_ENV]);
-	  rib_slot = (Free + RIB_NEXT_REDUCTION);
-	  Free += 3;
+	  (*rib_slot) = (MAKE_POINTER_OBJECT (rib_type, free));
+	  (free[RIB_EXP]) = (source_rib[RIB_EXP]);
+	  (free[RIB_ENV]) = (source_rib[RIB_ENV]);
+	  rib_slot = (free + RIB_NEXT_REDUCTION);
+	  free += 3;
 	  temp = (source_rib[RIB_NEXT_REDUCTION]);
 	  rib_type = (OBJECT_TYPE (temp));
 	  source_rib = (OBJECT_ADDRESS (temp));
@@ -828,6 +963,7 @@ copy_history (SCHEME_OBJECT hist_obj)
 	      (MAKE_POINTER_OBJECT
 	       ((OBJECT_TYPE (hist_ptr[HIST_PREV_SUBPROBLEM])),
 		last_hunk)));
+  Free = free;
   return (new_hunk);
 }
 
@@ -901,128 +1037,6 @@ record_primitive_entry (SCHEME_OBJECT primitive)
 
 #endif /* ENABLE_PRIMITIVE_PROFILING */
 
-/* Dynamic Winder support code */
-
-SCHEME_OBJECT
-Find_State_Space (SCHEME_OBJECT State_Point)
-{
-  long How_Far =
-    (UNSIGNED_FIXNUM_TO_LONG
-     (MEMORY_REF (State_Point, STATE_POINT_DISTANCE_TO_ROOT)));
-  long i;
-  SCHEME_OBJECT Point = State_Point;
-
-  for (i=0; i <= How_Far; i++)
-  {
-#ifdef ENABLE_DEBUGGING_TOOLS
-    if (Point == SHARP_F)
-    {
-      outf_fatal("\nState_Point %#lx wrong: count was %ld, #F at %ld\n",
-	      ((long) State_Point), ((long) How_Far), ((long) i));
-      Microcode_Termination(TERM_EXIT);
-      /*NOTREACHED*/
-    }
-#endif /* ENABLE_DEBUGGING_TOOLS */
-    Point = MEMORY_REF (Point, STATE_POINT_NEARER_POINT);
-  }
-  return (Point);
-}
-
-/* Assumptions:
-
-     (1) On a single processor, things should work with multiple state
-	 spaces.  The microcode variable current_state_point tracks
-	 the location in the "boot" space (i.e. the one whose space is
-	 #F) and the state spaces themselves (roots of the space
-	 trees) track the other spaces.
-     (2) On multi-processors, multiple spaces DO NOT work.  Only the
-	 initial space (#F) is tracked by the microcode (it is
-	 swapped on every task switch), but no association with trees
-	 is kept.  This will work since the initial tree has no space
-	 at the root, indicating that the microcode variable rather
-	 than the state space contains the current state space
-	 location.
-
-   NOTE: This procedure is invoked both by primitives and the interpreter
-   itself.  As such, it is using the pun that PRIMITIVE_ABORT is just a
-   (non-local) return to the interpreter.  This should be cleaned up.
-   NOTE: Any primitive that invokes this procedure must do a
-   canonicalize_primitive_context() first!  */
-
-void
-Translate_To_Point (SCHEME_OBJECT Target)
-{
-  SCHEME_OBJECT State_Space, Current_Location, *Path;
-  SCHEME_OBJECT Path_Point, *Path_Ptr;
-  long Distance, Merge_Depth, From_Depth, i;
-
-  State_Space = Find_State_Space(Target);
-  Path = Free;
-  Distance =
-    (UNSIGNED_FIXNUM_TO_LONG
-     (MEMORY_REF (Target, STATE_POINT_DISTANCE_TO_ROOT)));
-  if (State_Space == SHARP_F)
-    Current_Location = current_state_point;
-  else
-    Current_Location = MEMORY_REF (State_Space, STATE_SPACE_NEAREST_POINT);
-
-  if (Target == Current_Location)
-  {
-    PRIMITIVE_ABORT (PRIM_POP_RETURN);
-    /*NOTREACHED*/
-  }
-
-  for (Path_Ptr = (&(Path[Distance])), Path_Point = Target, i = 0;
-       i <= Distance;
-       i++)
-  {
-    *Path_Ptr-- = Path_Point;
-    Path_Point = MEMORY_REF (Path_Point, STATE_POINT_NEARER_POINT);
-  }
-
-  From_Depth =
-    (UNSIGNED_FIXNUM_TO_LONG
-     (MEMORY_REF (Current_Location, STATE_POINT_DISTANCE_TO_ROOT)));
-
-  for (Path_Point = Current_Location, Merge_Depth = From_Depth;
-       Merge_Depth > Distance;
-       Merge_Depth--)
-    Path_Point = MEMORY_REF (Path_Point, STATE_POINT_NEARER_POINT);
-
-  for (Path_Ptr = (&(Path[Merge_Depth]));
-       Merge_Depth >= 0;
-       Merge_Depth--, Path_Ptr--)
-  {
-    if (*Path_Ptr == Path_Point)
-      break;
-    Path_Point = MEMORY_REF (Path_Point, STATE_POINT_NEARER_POINT);
-  }
-
-#ifdef ENABLE_DEBUGGING_TOOLS
-  if (Merge_Depth < 0)
-  {
-    outf_fatal ("\nMerge_Depth went negative: %ld\n", Merge_Depth);
-    Microcode_Termination (TERM_EXIT);
-  }
-#endif /* ENABLE_DEBUGGING_TOOLS */
-
-  preserve_interrupt_mask ();
- Will_Push(CONTINUATION_SIZE + 4);
-  STACK_PUSH (LONG_TO_UNSIGNED_FIXNUM((Distance - Merge_Depth)));
-  STACK_PUSH (Target);
-  STACK_PUSH (LONG_TO_UNSIGNED_FIXNUM((From_Depth - Merge_Depth)));
-  STACK_PUSH (Current_Location);
-  SET_EXP (State_Space);
-  SET_RC(RC_MOVE_TO_ADJACENT_POINT);
-  SAVE_CONT();
- Pushed();
-
-  /* Disable lower than GC level */
-  SET_INTERRUPT_MASK (GET_INT_MASK & ((INT_GC << 1) - 1));
-  PRIMITIVE_ABORT (PRIM_POP_RETURN);
-  /*NOTREACHED*/
-}
-
 #ifdef __WIN32__
 
 #include <windows.h>
